#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <glob.h>
#include <string.h>
#include <libgen.h>
#include "file-system.h"

/**
 * Vraci jmeno existujiciho adresare (bez / na konci)
 * V pripade chyby vraci NULL, ale mela by vratit vzdy alespon "/"
 * @param dir Cesta, kde zacit hledat
 * @return odkaz na pole char delky PATH_MAX, nebo NULL v pripade chyby
 */
char *
getValidDirectory (char *dir)
{
  static char validDir[PATH_MAX];
  struct stat myStat;
  if (!dir)
    return NULL;
  strncpy (validDir, dir, PATH_MAX);
  do
    {
      if (stat (validDir, &myStat) || !S_ISDIR (myStat.st_mode))
	{
	  dir = dirname (validDir);
	  if (!dir)
	    return NULL;
	  strncpy (validDir, dir, PATH_MAX);
	  continue;
	}
      return validDir;
    }
  while (1);
}

fileInfo *
getDirectoryFiles (char *validDir)
{
  glob_t globbuf;
  int i;
  char *pattern;
  fileInfo *info;
  int len = strlen (validDir);
  pattern = (char *) malloc (sizeof (char) * (len + 3));
  if (!pattern)
    {
      return NULL;
    }
  strncpy (pattern, validDir, len + 1);
  strcat (pattern, "/*");
  if (glob (pattern, GLOB_MARK | GLOB_PERIOD, NULL, &globbuf))
    {
      free (pattern);
      return NULL;
    }
  info = (fileInfo *) malloc (sizeof (fileInfo) * (globbuf.gl_pathc + 1));
  if (!info)
    {
      globfree (&globbuf);
      free (pattern);
      return NULL;
    }
  for (i = 0; i < globbuf.gl_pathc; i++)
    {
      strncpy (info[i].path, globbuf.gl_pathv[i], PATH_MAX);
      if (lstat (globbuf.gl_pathv[i], &(info[i].stat)))
	{
	  globfree (&globbuf);
	  free (pattern);
	  free (info);
	  return NULL;
	}
      info[i].is_directory = S_ISDIR (info[i].stat.st_mode);
      info[i].is_link = S_ISLNK (info[i].stat.st_mode);
    }
  info[i].path[0] = '\0';

  globfree (&globbuf);
  free (pattern);
  return info;
}
